#include "dream.h"
#include "draw.h"

const int MAX_COLORS= 9;

const int black 	= 0;
const int white 	= 65535;
const int red 		= 64000;
const int orange	= 58887;
const int yellow 	= 57255;
const int green 	= 10000;
const int blue 		= 255;
const int indigo 	= 33333;
const int violet 	= 49500;

int canvas_color = 1;

mouse_cond_t mcond;
cont_cond_t cond;
uint8 mcont, mmouse;

void select_canvas(int *colors_ptr) {

	int c = 1;

	bfont_draw_str(vram_s + ( 40*640+10), 640, "Dream Paint ");
	bfont_draw_str(vram_s + (100*640+10), 640, "Use the mouse wheel to select a canvas color ");
	bfont_draw_str(vram_s + (130*640+10), 640, "Press left mouse button to start ");

	square(320,240,44,255);

	while (1) {
		/* Check for start on the controller */
		if (cont_get_cond(mcont, &cond)) {
			printf("Error getting controller status\r\n");
			return;
		}

		if (!(cond.buttons & CONT_START)) {
			printf("Pressed start\r\n");
			return;
		}

		/* Check for mouse input */
		if (mouse_get_cond(mmouse, &mcond)) {
			printf("Error checking mouse status\r\n");
			return;
		}

		/* Select canvas */
		c += mcond.dz;
		c = c < 0 ? 0 : c;
		c = c > MAX_COLORS - 1 ? MAX_COLORS - 1 : c;
		canvas_color = *(colors_ptr + c);

		filled_square (320,240,20,canvas_color);

		if (mcond.buttons == 251) {
			screen_color(canvas_color);
			return;
		}
	}
}



void dc_paint() {

	int draw_color;
	int cursor_color = 65535;
	int old_color = 0;
	int i = 0, x = 160, y = 120;
	int pixel_color;
	int colors[MAX_COLORS];
	int *colors_ptr = colors;
	int right_button = 0;
	int r = 0;
	int squarex = 0, squarey = 0;

	mcont = maple_controller_addr();
	if (!mcont) {
		printf("No controllers attached!\r\n");
		return;
	}
	mmouse = maple_mouse_addr();
	if (!mmouse) {
		printf("No mice attached!\r\n");
		return;
	}

	colors[0] = black;
	colors[1] = white;
	colors[2] = red;
	colors[3] = orange;
	colors[4] = yellow;
	colors[5] = green;
	colors[6] = blue;
	colors[7] = indigo;
	colors[8] = violet;

	select_canvas(colors_ptr);

	printf("Starting Main Loop\r\n");
	while (1) {
		/* Check for start on the controller */
		if (cont_get_cond(mcont, &cond)) {
			printf("Error getting controller status\r\n");
			return;
		}

		if (!(cond.buttons & CONT_START)) {
			printf("Pressed start\r\n");
			return;
		}

		/* Check for mouse input */
		if (mouse_get_cond(mmouse, &mcond)) {
			printf("Error checking mouse status\r\n");
			return;
		}

		/* Return to Menu */
		if (mcond.buttons == 247) {
			vid_clear(0,0,0);
			dc_paint();
		}

		/* Change color */
		i += mcond.dz;
		i = i < 0 ? 0 : i;
		i = i > MAX_COLORS - 1 ? MAX_COLORS - 1 : i;
		draw_color = colors[i];

		/* Draw */
		if (mcond.dx || mcond.dy) {
			if (mcond.buttons == 253)
			{
				if (!right_button)
				{
					right_button = 1;
					squarex = x;
					squarey = y;
				}
				square (squarex,squarey,r,canvas_color);
				r++;
				square (squarex,squarey,r,draw_color);
			}

			if ((mcond.buttons != 253) && (right_button))
			{
				right_button = 0;
				r = 0;
			}

			if (mcond.buttons != 251)
			{
				put_pixel(x,y,old_color);
			}
			x += mcond.dx;
			y += mcond.dy;
			pixel_color = mcond.buttons=251 ? draw_color : cursor_color;
			old_color=get_pixel(x,y);
			put_pixel(x,y,pixel_color);
		}

		/* Clear screen if both buttons pressed */
		if (mcond.buttons==249)
		{
			screen_color(canvas_color);
		}
	}
}

int dc_main() {

	/* Do initial setup */
	dc_setup(DM_640x480, PM_RGB565);

	dc_paint();

	return 0;
}
